# ND_CombatEngine.py
# Naval Duels File - Statistics and Battle Computation
# (c) Alan Gilfoy, 2007

# This program is intended to run the battles that comprise a Naval Duels game.


class Ship:
    def __init__(self, amount, name, power, armor, cost):
        self.amount = amount
        self.name = name
        self.power = power
        self.armor = armor
        self.cost = cost

class Fighter(Ship):
    def __init__(self):
        Ship.__init__(self, 0, 'Fighter', 9, 12, 95)
        self.targets = ['Fighter', 'Carrier', 'Frigate', 'Destroyer', 'Cruiser', 'Battleship', 'Submarine']\

class Submarine(Ship):
    def __init__(self):
        Ship.__init__(self, 0, 'Submarine', 100, 80, 1500)
        self.targets = ['Submarine', 'Carrier', 'Battleship', 'Destroyer', 'Cruiser', 'Frigate', 'Fighter']
    
class Destroyer(Ship):
    def __init__(self):
        Ship.__init__(self, 0, 'Destroyer', 200, 400, 1500)
        self.targets = ['Submarine', 'Destroyer', 'Frigate', 'Fighter', 'Cruiser', 'Battleship', 'Carrier']

class Frigate(Ship):
    def __init__(self):
        Ship.__init__(self, 0, 'Frigate', 100, 100, 1000)
        self.targets = ['Fighter', 'Destroyer', 'Frigate', 'Cruiser', 'Battleship', 'Carrier', 'Submarine']

class Cruiser(Ship):
    def __init__(self):
        Ship.__init__(self, 0, 'Cruiser', 400, 700, 2500)
        self.targets = ['Submarine', 'Cruiser', 'Destroyer', 'Frigate', 'Fighter', 'Carrier', 'Battleship']

class Battleship(Ship):
    def __init__(self):
        Ship.__init__(self, 0, 'Battleship', 900, 900, 3500)
        self.targets = ['Battleship', 'Carrier', 'Submarine', 'Cruiser', 'Destroyer', 'Frigate', 'Fighter']

class Carrier(Ship):
    def __init__(self):
        Ship.__init__(self, 0, 'Carrier', 75, 1000, 5000)
        self.targets = ['Fighter', 'Carrier', 'Submarine', 'Frigate', 'Destroyer', 'Cruiser', 'Battleship']

Fighter = Fighter()
Submarine = Submarine()
Destroyer = Destroyer()
Frigate = Frigate()
Cruiser = Cruiser()
Battleship = Battleship()
Carrier = Carrier()

# Firepower and armor are/will be used by the combat calculation.
#
# 'Cost' is not used yet, but I have a use planned for it:
# Game strategy will consist partially of choosing a fleet
# based on these ship costs and having X amount of resources available
# However, that is input-choosing and input-checking, rather than the battle calculation.

class Fleet:
    def __init__(self, Admiral=0):
        self.fi = Fighter.amount
        self.ca = Carrier.amount
        self.su = Submarine.amount
        self.fr = Frigate.amount
        self.de = Destroyer.amount
        self.cr = Cruiser.amount
        self.bs = Battleship.amount

        self.fi_label = "Carrier-Based Fighters"
        self.ca_label = "Aircraft Carriers"
        self.su_label = "Submarines"
        self.fr_label = "Frigates"
        self.de_label = "Destroyers"
        self.cr_label = "Cruisers"
        self.bs_label = "Battleships"
        
        self.Admiral = Admiral

    def number_of_ships(self):
        return self.fi + self.ca + self.su + self.fr + self.de + self.cr + self.bs

fleet_a = Fleet()
fleet_b = Fleet()
# Instances of the Fleet class, to make two "separate copies" of, it represent Fleet A and Fleet B.
# User interface program (NavalDuels.py) uses user input to adjust these instances of Fleet according to the input.

def display_the_fleets(fleet_a, fleet_b):
    '''
    Prints the various components of fleet A and fleet B (the amount of each shiptype present.)
    Displays the labels as plural or singular, as appropriate.
    '''
    if fleet_a.fi == 1:
        fleet_a.fi_label = "Carrier-Based Fighter"
    if fleet_a.ca == 1:
        fleet_a.ca_label = "Aircraft Carrier"
    if fleet_a.su == 1:
        fleet_a.su_label = "Submarine"
    if fleet_a.fr == 1:
        fleet_a.fr_label = "Frigate"
    if fleet_a.de == 1:
        fleet_a.de_label = "Destroyer"
    if fleet_a.cr == 1:
        fleet_a.cr_label = "Cruiser"
    if fleet_a.bs == 1:
        fleet_a.bs_label = "Battleship"

    if fleet_b.fi == 1:
        fleet_b.fi_label = "Carrier-Based Fighter"
    if fleet_b.ca == 1:
        fleet_b.ca_label = "Aircraft Carrier"
    if fleet_b.su == 1:
        fleet_b.su_label = "Submarine"
    if fleet_b.fr == 1:
        fleet_b.fr_label = "Frigate"
    if fleet_b.de == 1:
        fleet_b.de_label = "Destroyer"
    if fleet_b.cr == 1:
        fleet_b.cr_label = "Cruiser"
    if fleet_b.bs == 1:
        fleet_b.bs_label = "Battleship"
        
    print
    print "Fleet A"
    print fleet_a.fi, fleet_a.fi_label
    print fleet_a.ca, fleet_a.ca_label
    print fleet_a.su, fleet_a.su_label
    print fleet_a.fr, fleet_a.fr_label
    print fleet_a.de, fleet_a.de_label
    print fleet_a.cr, fleet_a.cr_label
    print fleet_a.bs, fleet_a.bs_label
    print
    print

    print "Fleet B"
    print fleet_b.fi, fleet_b.fi_label
    print fleet_b.ca, fleet_b.ca_label
    print fleet_b.su, fleet_b.su_label
    print fleet_b.fr, fleet_b.fr_label
    print fleet_b.de, fleet_b.de_label
    print fleet_b.cr, fleet_b.cr_label
    print fleet_b.bs, fleet_b.bs_label
    print
    print

def main_battle_calc(fleet_a, fleet_b):

    FiringOrder = [Fighter, Submarine, Destroyer, Frigate, Battleship, Cruiser, Carrier]

    for ship_type in FiringOrder:

        a_curr_amt = 0
        b_curr_amt = 0

        if ship_type == Fighter:
            a_curr_amt = fleet_a.fi
            b_curr_amt = fleet_b.fi

        if ship_type == Carrier:
            a_curr_amt = fleet_a.ca
            b_curr_amt = fleet_b.ca

        if ship_type == Submarine:
            a_curr_amt = fleet_a.su
            b_curr_amt = fleet_b.su

        if ship_type == Frigate:
            a_curr_amt = fleet_a.fr
            b_curr_amt = fleet_b.fr

        if ship_type == Destroyer:
            a_curr_amt = fleet_a.de
            b_curr_amt = fleet_b.de

        if ship_type == Cruiser:
            a_curr_amt = fleet_a.cr
            b_curr_amt = fleet_b.cr

        if ship_type == Battleship:
            a_curr_amt = fleet_a.bs
            b_curr_amt = fleet_b.bs

        # 'power' is shorthand for 'firepower'.

        for target_type in ship_type.targets:
            
            #print ship_type.name, "is currently firing at", target_type
            
            power_of_a = ship_type.power, ship_type.name, "FP, times", a_curr_amt, "Fleet A", ship_type.name, "s)"
            power_of_b = ship_type.power, ship_type.name, "FP, times", b_curr_amt, "Fleet B", ship_type.name, "s)"
            
            if target_type == 'Fighter':
                #print power_of_a
                #print "divided by", Fighter.armor, "which is the", target_type, "'s armor rating"
                #print "equals up to", (ship_type.power * a_curr_amt)/Fighter.armor, "B", target_type, "destroyed"
                #print
                #print power_of_b
                #print "divided by", Fighter.armor, "which is the", target_type, "'s armor rating"
                #print "equals up to", (ship_type.power * b_curr_amt)/Fighter.armor, "A", target_type, "destroyed"
                #print

                a_power = a_curr_amt * ship_type.power
                b_power = b_curr_amt * ship_type.power

                a_fi_killed = b_power / Fighter.armor
                b_fi_killed = a_power / Fighter.armor

                if a_fi_killed > fleet_a.fi: # you can't lose more ships than you have
                    fleet_a.fi = 0

                if a_fi_killed <= fleet_a.fi:
                    fleet_a.fi = fleet_a.fi - a_fi_killed

                if b_fi_killed > fleet_b.fi:
                    fleet_b.fi = 0

                if b_fi_killed <= fleet_b.fi:
                    fleet_b.fi = fleet_b.fi - b_fi_killed
                
            if target_type == 'Carrier':
                #print power_of_a
                #print "divided by", Carrier.armor, "which is the", target_type, "'s armor rating"
                #print "equals up to", (ship_type.power * a_curr_amt)/Carrier.armor, "B", target_type, "destroyed"
                #print
                #print power_of_b
                #print "divided by", Carrier.armor, "which is the", target_type, "'s armor rating"
                #print "equals up to", (ship_type.power * b_curr_amt)/Carrier.armor, "A", target_type, "destroyed"
                #print
                a_power = a_curr_amt * ship_type.power
                b_power = b_curr_amt * ship_type.power

                a_ca_killed = b_power / Carrier.armor
                b_ca_killed = a_power / Carrier.armor

                if a_ca_killed > fleet_a.ca:
                    fleet_a.ca = 0

                if a_ca_killed <= fleet_a.ca:
                    fleet_a.ca = fleet_a.ca - a_ca_killed

                if b_ca_killed > fleet_b.ca:
                    fleet_b.ca = 0

                if b_ca_killed <= fleet_b.ca:
                    fleet_b.ca = fleet_b.ca - b_ca_killed
                
            if target_type == 'Submarine':
                #print power_of_a
                #print "divided by", Submarine.armor, "which is the", target_type, "'s armor rating"
                #print "equals up to", (ship_type.power * a_curr_amt)/Submarine.armor, "B", target_type, "destroyed"

                #print
                #print power_of_b
                #print "divided by", Submarine.armor, "which is the", target_type, "'s armor rating"
                #print "equals up to", (ship_type.power * b_curr_amt)/Submarine.armor, "A", target_type, "destroyed"
                #print
                a_power = a_curr_amt * ship_type.power
                b_power = b_curr_amt * ship_type.power

                a_su_killed = b_power / Submarine.armor
                b_su_killed = a_power / Submarine.armor

                if a_su_killed > fleet_a.su:
                    fleet_a.su = 0

                if a_su_killed <= fleet_a.su:
                    fleet_a.su = fleet_a.su - a_su_killed

                if b_su_killed > fleet_b.su:
                    fleet_b.su = 0

                if b_su_killed <= fleet_b.su:
                    fleet_b.su = fleet_b.su - b_su_killed
                
            if target_type == 'Frigate':
                #print power_of_a
                #print "divided by", Frigate.armor, "which is the", target_type, "'s armor rating"
                #print "equals up to", (ship_type.power * a_curr_amt)/Frigate.armor, "B", target_type, "destroyed"
                #print
                #print power_of_b
                #print "divided by", Frigate.armor, "which is the", target_type, "'s armor rating"
                #print "equals up to", (ship_type.power * b_curr_amt)/Frigate.armor, "A", target_type, "destroyed"
                #print
                a_power = a_curr_amt * ship_type.power
                b_power = b_curr_amt * ship_type.power

                a_fr_killed = b_power / Frigate.armor
                b_fr_killed = a_power / Frigate.armor

                if a_fr_killed > fleet_a.fr:
                    fleet_a.fr = 0

                if a_fr_killed <= fleet_a.fr:
                    fleet_a.fr = fleet_a.fr - a_fr_killed

                if b_fr_killed > fleet_b.fr:
                    fleet_b.fr = 0

                if b_fr_killed <= fleet_b.fr:
                    fleet_b.fr = fleet_b.fr - b_fr_killed
                
            if target_type == 'Destroyer':
                #print power_of_a
                #print "divided by", Destroyer.armor, "which is the", target_type, "'s armor rating"
                #print "equals up to", (ship_type.power * a_curr_amt)/Destroyer.armor, "B", target_type, "destroyed"
                #print
                #print power_of_b
                #print "divided by", Destroyer.armor, "which is the", target_type, "'s armor rating"
                #print "equals up to", (ship_type.power * b_curr_amt)/Destroyer.armor, "A", target_type, "destroyed"
                #print
                a_power = a_curr_amt * ship_type.power
                b_power = b_curr_amt * ship_type.power

                a_de_killed = b_power / Destroyer.armor
                b_de_killed = a_power / Destroyer.armor

                if a_de_killed > fleet_a.de:
                    fleet_a.de = 0

                if a_de_killed <= fleet_a.de:
                    fleet_a.de = fleet_a.de - a_de_killed

                if b_de_killed > fleet_b.de:
                    fleet_b.de = 0

                if b_de_killed <= fleet_b.de:
                    fleet_b.de = fleet_b.de - b_de_killed
                    
            if target_type == 'Cruiser':
                #print power_of_a
                #print "divided by", Cruiser.armor, "which is the", target_type, "'s armor rating"
                #print "equals up to", (ship_type.power * a_curr_amt)/Cruiser.armor, "B", target_type, "destroyed"
                #print
                #print power_of_b
                #print "divided by", Cruiser.armor, "which is the", target_type, "'s armor rating"
                #print "equals up to", (ship_type.power * b_curr_amt)/Cruiser.armor, "A", target_type, "destroyed"
                #print
                a_power = a_curr_amt * ship_type.power
                b_power = b_curr_amt * ship_type.power

                a_cr_killed = b_power / Cruiser.armor
                b_cr_killed = a_power / Cruiser.armor

                if a_cr_killed > fleet_a.cr:
                    fleet_a.cr = 0

                if a_cr_killed <= fleet_a.cr:
                    fleet_a.cr = fleet_a.cr - a_cr_killed

                if b_cr_killed > fleet_b.cr:
                    fleet_b.cr = 0

                if b_cr_killed <= fleet_b.cr:
                    fleet_b.cr = fleet_b.cr - b_cr_killed
                
            if target_type == 'Battleship':
                #print power_of_a
                #print "divided by", Battleship.armor, "which is the", target_type, "'s armor rating"
                #print "equals up to", (ship_type.power * a_curr_amt)/Battleship.armor, "B", target_type, "destroyed"
                #print
                #print power_of_b
                #print "divided by", Battleship.armor, "which is the", target_type, "'s armor rating"
                #print "equals up to", (ship_type.power * b_curr_amt)/Battleship.armor, "A", target_type, "destroyed"
                #print
                a_power = a_curr_amt * ship_type.power
                b_power = b_curr_amt * ship_type.power

                a_bs_killed = b_power / Battleship.armor
                b_bs_killed = a_power / Battleship.armor

                if a_bs_killed > fleet_a.bs:
                    fleet_a.bs = 0

                if a_bs_killed <= fleet_a.bs:
                    fleet_a.bs = fleet_a.bs - a_bs_killed

                if b_bs_killed > fleet_b.bs:
                    fleet_b.bs = 0

                if b_bs_killed <= fleet_b.bs:
                    fleet_b.bs = fleet_b.bs - b_bs_killed

def find_winner(fleet_a, fleet_b):
    A_NoShips = False
    B_NoShips = False

    A_shipcount = fleet_a.number_of_ships()
    B_shipcount = fleet_b.number_of_ships()

    if A_shipcount == 0:
        A_NoShips = True

    if B_shipcount == 0:
        B_NoShips = True

    if A_NoShips and B_NoShips == False:
        winner = 2 # '2' represents Fleet B (B is the second letter in the alphabet)
        # simple, B wins if A is out of ships and it isn't.

    if A_NoShips == False and B_NoShips:
        winner = 1 # '1' represents Fleet A (A is the first letter in the alphabet)
        # simple, A wins if B is out of ships and it isn't.

    if A_NoShips == False and B_NoShips == False :
        winner = 0 # I'm using 0 to represent a draw/stalemate
        # a draw if both sides still have ships left
        
    return winner
    
def CombatEngine(fleet_a, fleet_b):

    # run_mbc is short for "run the main battle calc"
    hasbeenrun = 0
    run_mbc = True

    while run_mbc:
        main_battle_calc(fleet_a, fleet_b)
        hasbeenrun += 1
        if hasbeenrun == 5:
            run_mbc = False

    # main_battle_calc basically calls a for loop.
    # The for loop goes through a list (the firing order in this case) once and does certain things,
    # base don the item in the list that it's currently on.
    # But it doesn't go through that main list more than once.
    #
    # Sometimes, going through main_battle_calc's loop once won't eliminate one side's ships entirely.
    # So, five times is more likely to do it.

    winner = find_winner(fleet_a, fleet_b)

    return winner

if __name__ == '__main__':
    print "This is a module."
    print "This program is designed to be used by other programs."
    print "This particular module carries information used by the Naval Duels war-game."
    print
    import doctest
    doctest.testmod()
